@@ -25,17 +25,29 @@
 
 #define _GNU_SOURCE
 
+#include <glib.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h> /* for MAXHOSTNAMELEN */
+#include <sys/param.h>      /* for MAXHOSTNAMELEN */
 #else
 #define MAXHOSTNAMELEN 64
 #endif
-#include <sys/utsname.h>
+#ifdef HAVE_UTSNAME_DOMAINNAME
+#include <sys/utsname.h>    /* for uname() */
+#endif
 #include <sys/types.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>         /* Unix header for getpid() */
+#endif
+#ifdef G_OS_WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#include <process.h>
+#define getpid() _getpid()
+#endif
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
@@ -163,8 +175,10 @@ g_mime_utils_header_format_date (time_t date, int tz_offset)
 	
 	date += ((tz_offset / 100) * (60 * 60)) + (tz_offset % 100) * 60;
 	
-#ifdef HAVE_GMTIME_R
+#if defined (HAVE_GMTIME_R)
 	gmtime_r (&date, &tm);
+#elif defined (HAVE_GMTIME_S)
+	gmtime_s (&tm, &date);
 #else
 	memcpy (&tm, gmtime (&date), sizeof (tm));
 #endif
@@ -434,28 +448,38 @@ static time_t
 mktime_utc (struct tm *tm)
 {
 	time_t tt;
+	long tz;
 	
 	tm->tm_isdst = -1;
 	tt = mktime (tm);
 	
-#if defined (HAVE_TM_GMTOFF)
-	tt += tm->tm_gmtoff;
+#if defined (G_OS_WIN32)
+	_get_timezone (&tz);
+	if (tm->tm_isdst > 0) {
+		int dst;
+		
+		_get_dstbias (&dst);
+		tz += dst;
+	}
+#elif defined (HAVE_TM_GMTOFF)
+	tz = -tm->tm_gmtoff;
 #elif defined (HAVE_TIMEZONE)
 	if (tm->tm_isdst > 0) {
 #if defined (HAVE_ALTZONE)
-		tt -= altzone;
+		tz = altzone;
 #else /* !defined (HAVE_ALTZONE) */
-		tt -= (timezone - 3600);
+		tz = (timezone - 3600);
 #endif
-	} else
-		tt -= timezone;
+	} else {
+		tz = timezone;
+	}
 #elif defined (HAVE__TIMEZONE)
-	tt -= _timezone;
+	tz = _timezone;
 #else
 #error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
 #endif
 	
-	return tt;
+	return tt - tz;
 }
 
 static time_t
@@ -845,14 +869,14 @@ decode_addrspec (const char **in)
 	}
 	
 	addrspec = g_string_new ("");
-	g_string_append_len (addrspec, word, inptr - word);
+	g_string_append_len (addrspec, word, (size_t) (inptr - word));
 	
 	/* get the rest of the local-part */
 	decode_lwsp (&inptr);
 	while (*inptr == '.') {
 		g_string_append_c (addrspec, *inptr++);
 		if ((word = decode_word (&inptr))) {
-			g_string_append_len (addrspec, word, inptr - word);
+			g_string_append_len (addrspec, word, (size_t) (inptr - word));
 			decode_lwsp (&inptr);
 		} else {
 			w(g_warning ("Invalid local-part in addr-spec: %s", *in));
@@ -915,7 +939,7 @@ decode_msgid (const char **in)
 		while (*inptr && *inptr != '>')
 			inptr++;
 		
-		msgid = g_strndup (*in, inptr - *in);
+		msgid = g_strndup (*in, (size_t) (inptr - *in));
 		*in = inptr;
 	}
 	
@@ -1478,6 +1502,13 @@ charset_convert (iconv_t cd, const char *inbuf, size_t inleft, char **outp, size
 				break;
 			}
 			
+#ifdef G_OS_WIN32
+			/* seems that GnuWin32's libiconv 1.9 does not set errno in
+			 * the E2BIG case, so we have to fake it */
+			if (outleft <= inleft)
+				errno = E2BIG;
+#endif
+			
 			if (errno == E2BIG) {
 				/* need to grow the output buffer */
 				outlen += (inleft * 2) + 16;
@@ -1620,7 +1651,7 @@ g_mime_utils_decode_8bit (const char *text, size_t len)
 		
 		*outbuf++ = '\0';
 		
-		return g_realloc (out, outbuf - out);
+		return g_realloc (out, (size_t) (outbuf - out));
 	}
 	
 	outlen = charset_convert (cd, text, len, &out, &outleft, &ninval);
@@ -1664,7 +1695,7 @@ quoted_decode (const unsigned char *in, size_t len, unsigned char *out)
 		}
 	}
 	
-	return (outptr - out);
+	return (ssize_t) (outptr - out);
 }
 
 #define is_rfc2047_encoded_word(atom, len) (len >= 7 && !strncmp (atom, "=?", 2) && !strncmp (atom + len - 2, "?=", 2))
@@ -1695,14 +1726,21 @@ rfc2047_decode_word (const char *in, size_t inlen)
 	case 'B':
 	case 'b':
 		inptr += 2;
-		decoded = g_alloca (inend - inptr);
-		declen = g_mime_encoding_base64_decode_step (inptr, inend - inptr, decoded, &state, &save);
+		len = (size_t) (inend - inptr);
+		decoded = g_alloca (len);
+		declen = g_mime_encoding_base64_decode_step (inptr, len, decoded, &state, &save);
+		
+		if (declen == -1) {
+			d(fprintf (stderr, "encountered broken 'Q' encoding\n"));
+			return NULL;
+		}
 		break;
 	case 'Q':
 	case 'q':
 		inptr += 2;
-		decoded = g_alloca (inend - inptr);
-		declen = quoted_decode (inptr, inend - inptr, decoded);
+		len = (size_t) (inend - inptr);
+		decoded = g_alloca (len);
+		declen = quoted_decode (inptr, len, decoded);
 		
 		if (declen == -1) {
 			d(fprintf (stderr, "encountered broken 'Q' encoding\n"));
@@ -1736,10 +1774,10 @@ rfc2047_decode_word (const char *in, size_t inlen)
 		p = (char *) decoded;
 		len = declen;
 		
-		while (!g_utf8_validate (p, len, (const char **) &p)) {
-			len = declen - (p - (char *) decoded);
-			*p = '?';
-		}
+		//while (!g_utf8_validate (p, len, (const char **) &p)) {
+		//	len = declen - (p - (char *) decoded);
+		//	*p = '?';
+		//}
 		
 		return g_strndup ((char *) decoded, declen);
 	}
@@ -2369,11 +2407,11 @@ rfc2047_encode (const char *in, gushort safemask)
 		
 		switch (word->type) {
 		case WORD_ATOM:
-			g_string_append_len (out, word->start, word->end - word->start);
+			g_string_append_len (out, word->start, (size_t) (word->end - word->start));
 			break;
 		case WORD_QSTRING:
 			g_assert (safemask & IS_PSAFE);
-			g_string_append_len_quoted (out, word->start, word->end - word->start);
+			g_string_append_len_quoted (out, word->start, (size_t) (word->end - word->start));
 			break;
 		case WORD_2047:
 			if (prev && prev->type == WORD_2047) {
